// -------------------------------------------------------------------------------------------------
//          _____          _______                                   ____   _____
//      ,-:` \;',`'-,     |__  _(_)                                 |___ \ |  __ \
//    .'-;_,;  ':-;_,'.      | | _  _ __   _ __ ___    __ _  _ __     __) || |  | |
//   /;   '/    ,  _`.-\     | || || '_ \ | '_ ` _ \  / _` || '_ \   |__ < | |  | |
//  | '`. (`     /` ` \`|    | || || | | || | | | | || (_| || | | |  ___) || |__| |
//  |:.  `\`-.   \_   / |    |_||_||_| |_||_| |_| |_| \__,_||_| |_| |____/ |_____/
//  |     (   `,  .`\ ;'|
//   \     | .'     `-'/      Realtime terrain processing, rendering and analysis
//    `.   ;/        .'
//      `'-._____.-'`             http://www.tinman3d.com - info@tinman3d.com
// -------------------------------------------------------------------------------------------------
// FILE:       Configuration.hlsl
// -------------------------------------------------------------------------------------------------
// COPYRIGHT:  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// AUTHOR(s):  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// DISCLAIMER: This source  code is  provided "as  is" and  without warranties  as to performance or
//             merchantability. The  author and/or  distributors of  this source  code may have made
//             statements about this source code.  Any such statements do not  constitute warranties
//             and shall not be  relied on by  the user in deciding whether to use this source code.
//             This source code is provided without any expressed or implied warranties  whatsoever.
//             Because of the diversity of conditions and hardware under which this source code  may
//             be used,  no warranty  of fitness  for a  particular purpose  is offered. The user is
//             advised to test the source code thoroughly before relying on it. The user must assume
//             the entire risk of using the source code.
// -------------------------------------------------------------------------------------------------

#ifndef TINMAN__CONFIGURATION
#define TINMAN__CONFIGURATION

#include "../GpuX.hlsl"

// -------------------------------------------------------------------------------------------------
// Compile-time Configuration
// -------------------------------------------------------------------------------------------------

/*!
 * @brief Is seamless cubemap filtering supported?
 */
#ifndef CUBEMAP_SEAMLESS
#define CUBEMAP_SEAMLESS true
#endif

/*!
 * @brief Depicts the value range of the Z-component of normalized device coordinates (NDC):
 *        true : [0 .. 1]
 *        false : [-1 .. +1]
 */
#ifndef NEAR_AT_ZERO
#define NEAR_AT_ZERO true
#endif

/*!
 * @brief The offset from viewport coordinates (0,0) to the center of the corresponding pixel.
 */
#ifndef PIXEL_OFFSET
#define PIXEL_OFFSET 0.0
#endif

/*!
 * @brief Number of terrain materials to use in one pass.
 */
#ifndef TERRAIN_MATERIAL_COUNT
#define TERRAIN_MATERIAL_COUNT 4
#endif

/*!
 * @brief Enables debug mode for a tri-planar material texture.
 *
 * The following debug codes are supported:
 *
 * 0 : disable
 * 1 : planarX
 * 2 : planarY
 * 3 : planarZ
 *
 */
#ifndef TERRAIN_MATERIAL_DEBUG
#define TERRAIN_MATERIAL_DEBUG 0
#endif

/*!
 * @brief Use displacement mapping for terrain?
 */
#ifndef TERRAIN_MATERIAL_DISPLACEMENT
#define TERRAIN_MATERIAL_DISPLACEMENT true
#endif

/*!
 * @brief Are texture arrays available?
 */
#ifndef TERRAIN_TEXTURE_ARRAYS
#define TERRAIN_TEXTURE_ARRAYS true
#endif

/*!
 * @brief Enable rendering of thick lines?
 */
#ifndef THICK_LINES
#define THICK_LINES true
#endif

#endif
