// -------------------------------------------------------------------------------------------------
//          _____          _______                                   ____   _____
//      ,-:` \;',`'-,     |__  _(_)                                 |___ \ |  __ \
//    .'-;_,;  ':-;_,'.      | | _  _ __   _ __ ___    __ _  _ __     __) || |  | |
//   /;   '/    ,  _`.-\     | || || '_ \ | '_ ` _ \  / _` || '_ \   |__ < | |  | |
//  | '`. (`     /` ` \`|    | || || | | || | | | | || (_| || | | |  ___) || |__| |
//  |:.  `\`-.   \_   / |    |_||_||_| |_||_| |_| |_| \__,_||_| |_| |____/ |_____/
//  |     (   `,  .`\ ;'|
//   \     | .'     `-'/      Realtime terrain processing, rendering and analysis
//    `.   ;/        .'
//      `'-._____.-'`             http://www.tinman3d.com - info@tinman3d.com
// -------------------------------------------------------------------------------------------------
// FILE:       Demo.hlsl
// -------------------------------------------------------------------------------------------------
// COPYRIGHT:  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// AUTHOR(s):  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// DISCLAIMER: This source  code is  provided "as  is" and  without warranties  as to performance or
//             merchantability. The  author and/or  distributors of  this source  code may have made
//             statements about this source code.  Any such statements do not  constitute warranties
//             and shall not be  relied on by  the user in deciding whether to use this source code.
//             This source code is provided without any expressed or implied warranties  whatsoever.
//             Because of the diversity of conditions and hardware under which this source code  may
//             be used,  no warranty  of fitness  for a  particular purpose  is offered. The user is
//             advised to test the source code thoroughly before relying on it. The user must assume
//             the entire risk of using the source code.
// -------------------------------------------------------------------------------------------------

#ifndef TINMAN__DEMO__DEMO
#define TINMAN__DEMO__DEMO

#include "../../GpuX.hlsl"
#include "CB_Demo_Static.hlsl"
#include "SR_Demo.hlsl"

// -------------------------------------------------------------------------------------------------
// Prototypes
// -------------------------------------------------------------------------------------------------

float CellAt(float x, float y);

float CellCompute(float x, float y);

// -------------------------------------------------------------------------------------------------
// Functions
// -------------------------------------------------------------------------------------------------

float CellAt(float x, float y)
{
  float2 coords;
  float4 temp;

  coords = float2(x / g_bufferSize, y / g_bufferSize);
  temp = gxTexture2D(g_buffer, coords);

  // Return 0 for dead cells and 1 for alive cells.

  return abs(sign(temp.r));
}

float CellCompute(float x, float y)
{
  float alive;

  // Conway's Game of Life
  // https://en.wikipedia.org/wiki/Conway%27s_Game_of_Life
  // Every cell interacts with its eight neighbours, which are the cells that are horizontally,
  // vertically, or diagonally adjacent.

  alive = 0.0;
  alive += CellAt(x - 1.0, y);
  alive += CellAt(x + 1.0, y);
  alive += CellAt(x, y - 1.0);
  alive += CellAt(x, y + 1.0);
  alive += CellAt(x - 1.0, y - 1.0);
  alive += CellAt(x + 1.0, y - 1.0);
  alive += CellAt(x - 1.0, y + 1.0);
  alive += CellAt(x + 1.0, y + 1.0);

  if (CellAt(x, y) != 0.0)
  {
    // "Any live cell with fewer than two live neighbours dies, as if by underpopulation."

    if (alive < 2.0)
      return 0.0;

    // "Any live cell with more than three live neighbours dies, as if by overpopulation."

    if (alive > 3.0)
      return 0.0;

    // "Any live cell with two or three live neighbours lives on to the next generation."

    return 1.0;
  }

  // "Any dead cell with exactly three live neighbours becomes a live cell, as if by reproduction."

  if (alive == 3.0)
    return 1.0;

  return 0.0;
}

#endif
