// -------------------------------------------------------------------------------------------------
//          _____          _______                                   ____   _____
//      ,-:` \;',`'-,     |__  _(_)                                 |___ \ |  __ \
//    .'-;_,;  ':-;_,'.      | | _  _ __   _ __ ___    __ _  _ __     __) || |  | |
//   /;   '/    ,  _`.-\     | || || '_ \ | '_ ` _ \  / _` || '_ \   |__ < | |  | |
//  | '`. (`     /` ` \`|    | || || | | || | | | | || (_| || | | |  ___) || |__| |
//  |:.  `\`-.   \_   / |    |_||_||_| |_||_| |_| |_| \__,_||_| |_| |____/ |_____/
//  |     (   `,  .`\ ;'|
//   \     | .'     `-'/      Realtime terrain processing, rendering and analysis
//    `.   ;/        .'
//      `'-._____.-'`             http://www.tinman3d.com - info@tinman3d.com
// -------------------------------------------------------------------------------------------------
// FILE:       CB_Graphics_Static.hlsl
// -------------------------------------------------------------------------------------------------
// COPYRIGHT:  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// AUTHOR(s):  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// DISCLAIMER: This source  code is  provided "as  is" and  without warranties  as to performance or
//             merchantability. The  author and/or  distributors of  this source  code may have made
//             statements about this source code.  Any such statements do not  constitute warranties
//             and shall not be  relied on by  the user in deciding whether to use this source code.
//             This source code is provided without any expressed or implied warranties  whatsoever.
//             Because of the diversity of conditions and hardware under which this source code  may
//             be used,  no warranty  of fitness  for a  particular purpose  is offered. The user is
//             advised to test the source code thoroughly before relying on it. The user must assume
//             the entire risk of using the source code.
// -------------------------------------------------------------------------------------------------

#ifndef TINMAN__GRAPHICS__CB_GRAPHICS_STATIC
#define TINMAN__GRAPHICS__CB_GRAPHICS_STATIC

#include "../../GpuX.hlsl"

// -------------------------------------------------------------------------------------------------
// Constant Buffers
// -------------------------------------------------------------------------------------------------

cbuffer CB_Graphics_Static
{
  /*!
   * @brief The screen mask coefficients:
   *
   *        X := minimum screen X-coordinate of mask bounds
   *        Y := minimum screen Y-coordinate of mask bounds
   *        Z := screen width of mask bounds, in pixels
   *        W := screen height of mask bounds, in pixels
   *
   *
   * The screen mask is active iff Z is greater than zero.
   */
  float4 g_screenMask;

  /*!
   * @brief The line thickness coefficients:
   *
   *        X := (thickness + 1) / width
   *        Y := (thickness + 1) / height
   *        Z := (thickness + 1) / (thickness - 1) - 1
   *
   *
   * Thick line rendering is active iff Z is greater than zero.
   */
  float3 g_lineThickness;

  /*!
   * @brief The viewport size, in pixels.
   */
  float2 g_screenSize;
};

#endif
