// -------------------------------------------------------------------------------------------------
//          _____          _______                                   ____   _____
//      ,-:` \;',`'-,     |__  _(_)                                 |___ \ |  __ \
//    .'-;_,;  ':-;_,'.      | | _  _ __   _ __ ___    __ _  _ __     __) || |  | |
//   /;   '/    ,  _`.-\     | || || '_ \ | '_ ` _ \  / _` || '_ \   |__ < | |  | |
//  | '`. (`     /` ` \`|    | || || | | || | | | | || (_| || | | |  ___) || |__| |
//  |:.  `\`-.   \_   / |    |_||_||_| |_||_| |_| |_| \__,_||_| |_| |____/ |_____/
//  |     (   `,  .`\ ;'|
//   \     | .'     `-'/      Realtime terrain processing, rendering and analysis
//    `.   ;/        .'
//      `'-._____.-'`             http://www.tinman3d.com - info@tinman3d.com
// -------------------------------------------------------------------------------------------------
// FILE:       Model.PS_Shadow.hlsl
// -------------------------------------------------------------------------------------------------
// COPYRIGHT:  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// AUTHOR(s):  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// DISCLAIMER: This source  code is  provided "as  is" and  without warranties  as to performance or
//             merchantability. The  author and/or  distributors of  this source  code may have made
//             statements about this source code.  Any such statements do not  constitute warranties
//             and shall not be  relied on by  the user in deciding whether to use this source code.
//             This source code is provided without any expressed or implied warranties  whatsoever.
//             Because of the diversity of conditions and hardware under which this source code  may
//             be used,  no warranty  of fitness  for a  particular purpose  is offered. The user is
//             advised to test the source code thoroughly before relying on it. The user must assume
//             the entire risk of using the source code.
// -------------------------------------------------------------------------------------------------

#ifndef TINMAN__MODEL__MODEL_PS_SHADOW
#define TINMAN__MODEL__MODEL_PS_SHADOW

#include "../../GpuX.hlsl"
#include "../PS_Out_TargetDepth.hlsl"
#include "../Shadow.hlsl"
#include "CB_Material_Dynamic1.hlsl"
#include "Model.hlsl"
#include "VS_Out_Shadow.hlsl"

// -------------------------------------------------------------------------------------------------
// Pixel Shader 'Model.PS_Shadow'
// -------------------------------------------------------------------------------------------------

PS_Out_TargetDepth main(centroid VS_Out_Shadow data)
{
  PS_Out_TargetDepth result;
  float a;

  // Apply distance fade range.

  a = ModelFade(data.Distance);

  clip(a - 0.002);

  // Apply model opacity.

  a *= MaterialAlpha(data.Color.a * g_materialBaseColor.a * Texture_RGBA(MaterialBaseColor, data.
      Coords).a, data.Coords);

  clip(a - 0.002);

  // Output interpolated linear Z.

  result.Color = WriteShadow(data.Depth.x / data.Depth.y, a).Color;
  result.Depth = result.Color.x;

  return result;
}

#endif
