// -------------------------------------------------------------------------------------------------
//          _____          _______                                   ____   _____
//      ,-:` \;',`'-,     |__  _(_)                                 |___ \ |  __ \
//    .'-;_,;  ':-;_,'.      | | _  _ __   _ __ ___    __ _  _ __     __) || |  | |
//   /;   '/    ,  _`.-\     | || || '_ \ | '_ ` _ \  / _` || '_ \   |__ < | |  | |
//  | '`. (`     /` ` \`|    | || || | | || | | | | || (_| || | | |  ___) || |__| |
//  |:.  `\`-.   \_   / |    |_||_||_| |_||_| |_| |_| \__,_||_| |_| |____/ |_____/
//  |     (   `,  .`\ ;'|
//   \     | .'     `-'/      Realtime terrain processing, rendering and analysis
//    `.   ;/        .'
//      `'-._____.-'`             http://www.tinman3d.com - info@tinman3d.com
// -------------------------------------------------------------------------------------------------
// FILE:       Model.PS_Transparent.hlsl
// -------------------------------------------------------------------------------------------------
// COPYRIGHT:  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// AUTHOR(s):  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// DISCLAIMER: This source  code is  provided "as  is" and  without warranties  as to performance or
//             merchantability. The  author and/or  distributors of  this source  code may have made
//             statements about this source code.  Any such statements do not  constitute warranties
//             and shall not be  relied on by  the user in deciding whether to use this source code.
//             This source code is provided without any expressed or implied warranties  whatsoever.
//             Because of the diversity of conditions and hardware under which this source code  may
//             be used,  no warranty  of fitness  for a  particular purpose  is offered. The user is
//             advised to test the source code thoroughly before relying on it. The user must assume
//             the entire risk of using the source code.
// -------------------------------------------------------------------------------------------------

#ifndef TINMAN__MODEL__MODEL_PS_TRANSPARENT
#define TINMAN__MODEL__MODEL_PS_TRANSPARENT

#include "../../GpuX.hlsl"
#include "../PS_Out_DualTargetDepth.hlsl"
#include "../PS_Out_TargetDepthLess.hlsl"
#include "Model.hlsl"
#include "VS_Out.hlsl"

// -------------------------------------------------------------------------------------------------
// Pixel Shader 'Model.PS_Transparent'
// -------------------------------------------------------------------------------------------------

PS_Out_DualTargetDepth main(centroid VS_Out data, FACE face)
{
  PS_Out_DualTargetDepth result;
  PS_Out_TargetDepthLess pixel;
  float weight;

  pixel = ModelPixel(data, face);

  // Weight by opacity and logarithmic depth.

  weight = pixel.Color.a * (1.0 - pixel.Depth);

  // The weighted color will be summed:
  // C_dst = C_dst + C_src
  // The alpha values will be accumulated:
  // A_dst = A_dst * (1 - A_src)

  result.ColorA = float4(pixel.Color.rgb * weight, pixel.Color.a);
  result.ColorB = float4(weight, 0, 0, 0);
  result.Depth = pixel.Depth;

  return result;
}

#endif
