// -------------------------------------------------------------------------------------------------
//          _____          _______                                   ____   _____
//      ,-:` \;',`'-,     |__  _(_)                                 |___ \ |  __ \
//    .'-;_,;  ':-;_,'.      | | _  _ __   _ __ ___    __ _  _ __     __) || |  | |
//   /;   '/    ,  _`.-\     | || || '_ \ | '_ ` _ \  / _` || '_ \   |__ < | |  | |
//  | '`. (`     /` ` \`|    | || || | | || | | | | || (_| || | | |  ___) || |__| |
//  |:.  `\`-.   \_   / |    |_||_||_| |_||_| |_| |_| \__,_||_| |_| |____/ |_____/
//  |     (   `,  .`\ ;'|
//   \     | .'     `-'/      Realtime terrain processing, rendering and analysis
//    `.   ;/        .'
//      `'-._____.-'`             http://www.tinman3d.com - info@tinman3d.com
// -------------------------------------------------------------------------------------------------
// FILE:       VS_Out.hlsl
// -------------------------------------------------------------------------------------------------
// COPYRIGHT:  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// AUTHOR(s):  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// DISCLAIMER: This source  code is  provided "as  is" and  without warranties  as to performance or
//             merchantability. The  author and/or  distributors of  this source  code may have made
//             statements about this source code.  Any such statements do not  constitute warranties
//             and shall not be  relied on by  the user in deciding whether to use this source code.
//             This source code is provided without any expressed or implied warranties  whatsoever.
//             Because of the diversity of conditions and hardware under which this source code  may
//             be used,  no warranty  of fitness  for a  particular purpose  is offered. The user is
//             advised to test the source code thoroughly before relying on it. The user must assume
//             the entire risk of using the source code.
// -------------------------------------------------------------------------------------------------

#ifndef TINMAN__MODEL__VS_OUT
#define TINMAN__MODEL__VS_OUT

#include "../../GpuX.hlsl"

// -------------------------------------------------------------------------------------------------
// Shader Parameters
// -------------------------------------------------------------------------------------------------

struct VS_Out
{
  /*!
   * @brief Base color and opacity
   */
  float4 Color SEMANTIC_COLOR(0);

  /*!
   * @brief Texture coordinate sets, used implicitly via MaterialTexture_*(...)
   */
  float4 Coords SEMANTIC_TEXCOORD(1);

  /*!
   * @brief Position value
   */
  float4 Plain SEMANTIC_TEXCOORD(3);

  /*!
   * @brief Position value
   */
  float4 Position SEMANTIC_SYSTEM_POSITION(0);

  /*!
   * @brief Shadow coefficients, to be used with 'Shadow.ComputeShadow'.
   */
  float4 Shadow SEMANTIC_TEXCOORD(4);

  /*!
   * @brief Bitangent vector
   */
  float3 Bitangent SEMANTIC_TANGENT(0);

  /*!
   * @brief Eye vector
   */
  float3 Eye SEMANTIC_NORMAL(0);

  /*!
   * @brief Normal vector
   */
  float3 Normal SEMANTIC_NORMAL(1);

  /*!
   * @brief Tangent vector
   */
  float3 Tangent SEMANTIC_TANGENT(1);

  /*!
   * @brief Line thickness offset vector
   */
  noperspective float2 Clip SEMANTIC_TEXCOORD(0);

  /*!
   * @brief Distance from camera (x) Ambient occlusion as scale factor for ambient light (y)
   */
  float2 DistanceOcclusion SEMANTIC_TEXCOORD(2);
};

#endif
