// -------------------------------------------------------------------------------------------------
//          _____          _______                                   ____   _____
//      ,-:` \;',`'-,     |__  _(_)                                 |___ \ |  __ \
//    .'-;_,;  ':-;_,'.      | | _  _ __   _ __ ___    __ _  _ __     __) || |  | |
//   /;   '/    ,  _`.-\     | || || '_ \ | '_ ` _ \  / _` || '_ \   |__ < | |  | |
//  | '`. (`     /` ` \`|    | || || | | || | | | | || (_| || | | |  ___) || |__| |
//  |:.  `\`-.   \_   / |    |_||_||_| |_||_| |_| |_| \__,_||_| |_| |____/ |_____/
//  |     (   `,  .`\ ;'|
//   \     | .'     `-'/      Realtime terrain processing, rendering and analysis
//    `.   ;/        .'
//      `'-._____.-'`             http://www.tinman3d.com - info@tinman3d.com
// -------------------------------------------------------------------------------------------------
// FILE:       Terrain.PS_Material.hlsl
// -------------------------------------------------------------------------------------------------
// COPYRIGHT:  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// AUTHOR(s):  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// DISCLAIMER: This source  code is  provided "as  is" and  without warranties  as to performance or
//             merchantability. The  author and/or  distributors of  this source  code may have made
//             statements about this source code.  Any such statements do not  constitute warranties
//             and shall not be  relied on by  the user in deciding whether to use this source code.
//             This source code is provided without any expressed or implied warranties  whatsoever.
//             Because of the diversity of conditions and hardware under which this source code  may
//             be used,  no warranty  of fitness  for a  particular purpose  is offered. The user is
//             advised to test the source code thoroughly before relying on it. The user must assume
//             the entire risk of using the source code.
// -------------------------------------------------------------------------------------------------

#ifndef TINMAN__TERRAIN__TERRAIN_PS_MATERIAL
#define TINMAN__TERRAIN__TERRAIN_PS_MATERIAL

#include "../../GpuX.hlsl"
#include "../Configuration.hlsl"
#include "../PS_Out_Target.hlsl"
#include "../Shadow.hlsl"
#include "CB_Material_Static.hlsl"
#include "CB_Material_Volatile1.hlsl"
#include "Material.hlsl"
#include "PS_In_Material.hlsl"
#include "Terrain.hlsl"

// -------------------------------------------------------------------------------------------------
// Pixel Shader 'Terrain.PS_Material'
// -------------------------------------------------------------------------------------------------

PS_Out_Target main(PS_In_Material data, FACE face)
{
  PS_Out_Target result;
  float blend;
  float shadow;
  float2x2 rotation;
  float3 temp;
  float weightOriginal;
  float weightScaled;

  // Compute layer blend and clip pixel if possible.

  blend = LayerBlendClip(data.Distance);
  data.Eye = normalize(data.Eye);
  data.MaterialNormal = normalize(data.MaterialNormal);
  shadow = ComputeShadow(data.Shadow);
  rotation = MaterialRotation();

  // Perform tri-planar terrain texturing with up to four materials.

  weightOriginal = weightScaled = 0.0;
  temp = 0;

  for (int i = 0; i < TERRAIN_MATERIAL_COUNT; ++i)
  {
    if (g_materialWeights[i] > 0.0)
    {
      Material material;
      float weight;

      material = MaterialSample(i, data.MaterialCoords, data.MaterialNormal, data.Color);
      material.Normal.xz = mul(material.Normal.xz, rotation);
      material.Normal *= -gxFace(face);
      material.Occlusion *= data.Occlusion;
      weight = data.MaterialWeights[i];
      weightOriginal += weight;
      weight = MaterialBlendWeight(material.Height, weight);
      weightScaled += weight;
      temp += MaterialColor(material, data.Eye, data.Distance, shadow) * weight;
    }
  }

  // Scale to original material weight sum.

  temp *= weightOriginal / weightScaled;

  if (g_materialGridLines > 0.0)
  {
    temp = MaterialGridLine(data.MaterialCoords.x, temp, float3(1, 0, 0));
    temp = MaterialGridLine(data.MaterialCoords.y, temp, float3(0, 1, 0));
    temp = MaterialGridLine(data.MaterialCoords.z, temp, float3(0, 0, 1));
  }

  result.Color = MaterialBlend(temp, blend);

  return result;
}

#endif
