// -------------------------------------------------------------------------------------------------
//          _____          _______                                   ____   _____
//      ,-:` \;',`'-,     |__  _(_)                                 |___ \ |  __ \
//    .'-;_,;  ':-;_,'.      | | _  _ __   _ __ ___    __ _  _ __     __) || |  | |
//   /;   '/    ,  _`.-\     | || || '_ \ | '_ ` _ \  / _` || '_ \   |__ < | |  | |
//  | '`. (`     /` ` \`|    | || || | | || | | | | || (_| || | | |  ___) || |__| |
//  |:.  `\`-.   \_   / |    |_||_||_| |_||_| |_| |_| \__,_||_| |_| |____/ |_____/
//  |     (   `,  .`\ ;'|
//   \     | .'     `-'/      Realtime terrain processing, rendering and analysis
//    `.   ;/        .'
//      `'-._____.-'`             http://www.tinman3d.com - info@tinman3d.com
// -------------------------------------------------------------------------------------------------
// FILE:       Terrain.PS_Texture.hlsl
// -------------------------------------------------------------------------------------------------
// COPYRIGHT:  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// AUTHOR(s):  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// DISCLAIMER: This source  code is  provided "as  is" and  without warranties  as to performance or
//             merchantability. The  author and/or  distributors of  this source  code may have made
//             statements about this source code.  Any such statements do not  constitute warranties
//             and shall not be  relied on by  the user in deciding whether to use this source code.
//             This source code is provided without any expressed or implied warranties  whatsoever.
//             Because of the diversity of conditions and hardware under which this source code  may
//             be used,  no warranty  of fitness  for a  particular purpose  is offered. The user is
//             advised to test the source code thoroughly before relying on it. The user must assume
//             the entire risk of using the source code.
// -------------------------------------------------------------------------------------------------

#ifndef TINMAN__TERRAIN__TERRAIN_PS_TEXTURE
#define TINMAN__TERRAIN__TERRAIN_PS_TEXTURE

#include "../../GpuX.hlsl"
#include "../CB_Lighting_Static.hlsl"
#include "../PS_Out_Target.hlsl"
#include "../Shadow.hlsl"
#include "CB_Texture_Static.hlsl"
#include "Material.hlsl"
#include "PS_In_Texture.hlsl"
#include "Terrain.hlsl"

// -------------------------------------------------------------------------------------------------
// Pixel Shader 'Terrain.PS_Texture'
// -------------------------------------------------------------------------------------------------

PS_Out_Target main(PS_In_Texture data, FACE face)
{
  PS_Out_Target result;
  Material material;
  float opacity;
  float blend;

  blend = LayerBlendClip(data.EyeDistance.w);
  material = TextureSample(data.Color, data.TexCoords, data.EyeDistance.w, opacity);

  if (g_terrainData > 0.0)
  {
    // Per-vertex normal are already in light-space.

    material.Normal = normalize(data.Normal);
  }
  else
  {
    // Convert normal vector from terrain-space to light-space.

    material.Normal = normalize(mul(g_lightSpace, material.Normal));
  }

  material.Normal *= -gxFace(face);

  // Compute final color.

  result.Color = Highlight(float4(MaterialColor(material, normalize(data.EyeDistance.xyz), data.
      EyeDistance.w, ComputeShadow(data.Shadow)), saturate(blend) * opacity));

  return result;
}

#endif
