// -------------------------------------------------------------------------------------------------
//          _____          _______                                   ____   _____
//      ,-:` \;',`'-,     |__  _(_)                                 |___ \ |  __ \
//    .'-;_,;  ':-;_,'.      | | _  _ __   _ __ ___    __ _  _ __     __) || |  | |
//   /;   '/    ,  _`.-\     | || || '_ \ | '_ ` _ \  / _` || '_ \   |__ < | |  | |
//  | '`. (`     /` ` \`|    | || || | | || | | | | || (_| || | | |  ___) || |__| |
//  |:.  `\`-.   \_   / |    |_||_||_| |_||_| |_| |_| \__,_||_| |_| |____/ |_____/
//  |     (   `,  .`\ ;'|
//   \     | .'     `-'/      Realtime terrain processing, rendering and analysis
//    `.   ;/        .'
//      `'-._____.-'`             http://www.tinman3d.com - info@tinman3d.com
// -------------------------------------------------------------------------------------------------
// FILE:       Terrain.TP_Material.hlsl
// -------------------------------------------------------------------------------------------------
// COPYRIGHT:  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// AUTHOR(s):  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// DISCLAIMER: This source  code is  provided "as  is" and  without warranties  as to performance or
//             merchantability. The  author and/or  distributors of  this source  code may have made
//             statements about this source code.  Any such statements do not  constitute warranties
//             and shall not be  relied on by  the user in deciding whether to use this source code.
//             This source code is provided without any expressed or implied warranties  whatsoever.
//             Because of the diversity of conditions and hardware under which this source code  may
//             be used,  no warranty  of fitness  for a  particular purpose  is offered. The user is
//             advised to test the source code thoroughly before relying on it. The user must assume
//             the entire risk of using the source code.
// -------------------------------------------------------------------------------------------------

#ifndef TINMAN__TERRAIN__TERRAIN_TP_MATERIAL
#define TINMAN__TERRAIN__TERRAIN_TP_MATERIAL

#include "../../GpuX.hlsl"
#include "../TS_Patch.hlsl"
#include "DisplacementVarying.hlsl"
#include "GS_In_Material.hlsl"
#include "Terrain.hlsl"

// -------------------------------------------------------------------------------------------------
// Tessellate Patch Shader 'Terrain.TP_Material'
// -------------------------------------------------------------------------------------------------

[domain("tri")]
[outputcontrolpoints(3)]
[outputtopology("triangle_cw")]
[partitioning("integer")]
[patchconstantfunc("main_patch")]
GS_In_Material main(InputPatch<GS_In_Material,3> points, uint index : SV_OutputControlPointID)
{
  GS_In_Material result;

  result = points[index];

  return result;
}

TS_Patch main_patch(InputPatch<GS_In_Material,3> points)
{
  TS_Patch data;
  {
    DisplacementVarying displacement[3];

    displacement[0] = ComputeDisplacementVarying(points[0].Fixed, points[0].WeightsA, true);
    displacement[1] = ComputeDisplacementVarying(points[1].Fixed, points[1].WeightsA, true);
    displacement[2] = ComputeDisplacementVarying(points[2].Fixed, points[2].WeightsA, true);
    data.FactorOuter[0] = MaterialTessellationFactor(points[1], points[2], displacement[1],
        displacement[2]);
    data.FactorOuter[1] = MaterialTessellationFactor(points[2], points[0], displacement[2],
        displacement[0]);
    data.FactorOuter[2] = MaterialTessellationFactor(points[0], points[1], displacement[0],
        displacement[1]);
    data.FactorInner = (data.FactorOuter[0] + data.FactorOuter[1] + data.FactorOuter[2]) / 3.0;
  }
  return data;
}

#endif
