// -------------------------------------------------------------------------------------------------
//          _____          _______                                   ____   _____
//      ,-:` \;',`'-,     |__  _(_)                                 |___ \ |  __ \
//    .'-;_,;  ':-;_,'.      | | _  _ __   _ __ ___    __ _  _ __     __) || |  | |
//   /;   '/    ,  _`.-\     | || || '_ \ | '_ ` _ \  / _` || '_ \   |__ < | |  | |
//  | '`. (`     /` ` \`|    | || || | | || | | | | || (_| || | | |  ___) || |__| |
//  |:.  `\`-.   \_   / |    |_||_||_| |_||_| |_| |_| \__,_||_| |_| |____/ |_____/
//  |     (   `,  .`\ ;'|
//   \     | .'     `-'/      Realtime terrain processing, rendering and analysis
//    `.   ;/        .'
//      `'-._____.-'`             http://www.tinman3d.com - info@tinman3d.com
// -------------------------------------------------------------------------------------------------
// FILE:       Terrain.VS_Material1.hlsl
// -------------------------------------------------------------------------------------------------
// COPYRIGHT:  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// AUTHOR(s):  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// DISCLAIMER: This source  code is  provided "as  is" and  without warranties  as to performance or
//             merchantability. The  author and/or  distributors of  this source  code may have made
//             statements about this source code.  Any such statements do not  constitute warranties
//             and shall not be  relied on by  the user in deciding whether to use this source code.
//             This source code is provided without any expressed or implied warranties  whatsoever.
//             Because of the diversity of conditions and hardware under which this source code  may
//             be used,  no warranty  of fitness  for a  particular purpose  is offered. The user is
//             advised to test the source code thoroughly before relying on it. The user must assume
//             the entire risk of using the source code.
// -------------------------------------------------------------------------------------------------

#ifndef TINMAN__TERRAIN__TERRAIN_VS_MATERIAL1
#define TINMAN__TERRAIN__TERRAIN_VS_MATERIAL1

#include "../../GpuX.hlsl"
#include "../Camera.hlsl"
#include "../Util.hlsl"
#include "CB_Material_Volatile1.hlsl"
#include "Displacement.hlsl"
#include "DisplacementFixed.hlsl"
#include "DisplacementVarying.hlsl"
#include "PS_In_Material.hlsl"
#include "Terrain.hlsl"
#include "VS_In.hlsl"

// -------------------------------------------------------------------------------------------------
// Vertex Shader 'Terrain.VS_Material1'
// -------------------------------------------------------------------------------------------------

PS_In_Material main(VS_In data)
{
  PS_In_Material result;
  Displacement displacement;

  displacement = ComputeDisplacement(data);
  result.Distance = displacement.Fixed.Distance;
  result.Normal = displacement.Fixed.Normal;
  result.Eye = displacement.Fixed.Eye;
  result.MaterialCoords = displacement.Fixed.MaterialCoords;
  result.MaterialNormal = displacement.Fixed.MaterialNormal;
  result.Occlusion = data.Color.a;
  result.Shadow = displacement.Varying.Shadow;
  result.Color = FromSrgb(data.Color.rgb);
  result.Position = ProjectPosition(displacement.Varying.Position);
  result.MaterialWeights = MaterialComputeWeights(displacement.Fixed.MaterialIds, data.
      MaterialWeights, g_materialIds);

  return result;
}

#endif
